#!/usr/bin/env python

import os
from dm.common.exceptions.invalidRequest import InvalidRequest
from dm.common.exceptions.objectNotFound import ObjectNotFound
from dm.common.exceptions.invalidArgument import InvalidArgument
from dm.common.cli.dmCli import DmCli
from dm.common.utility.ldapUserManager import LdapUserManager
from dm.common.utility.configurationManager import ConfigurationManager
from dm.aps_user_db.api.apsUserDbApi import ApsUserDbApi

class UpdateUserFromApsDbCli(DmCli):
    def __init__(self):
        DmCli.__init__(self)
        self.addOption('', '--badge', dest='badge', help='User badge number.')
        self.addOption('', '--config-file', dest='configFile', help='Utility configuration file.')

    def checkArgs(self):
        if self.options.badge is None:
            raise InvalidRequest('Badge number must be provided.')
        configFile = self.options.configFile
        if configFile is None:
            raise InvalidRequest('Config file must be provided.')
        if not os.path.exists(configFile):
            raise InvalidArgument('Invalid config file: %s.', configFile)

    def getBadge(self):
        return self.options.badge

    def runCommand(self):
        self.parseArgs(usage="""
    dm-update-user-from-aps-db --badge=BADGE --config-file=CONFIGFILE

Description:
    Updates user in DM LDAP and database using information from APS DB. 
        """)
        ConfigurationManager.getInstance().setConfigFile(self.options.configFile)
        self.checkArgs()
        apsUserDbApi = ApsUserDbApi()
        badge = self.getBadge()
        username = 'd%s' % badge
        apsUser = apsUserDbApi.getApsUserByBadgeNumber(badge)
        #print 'APS User Info: ', apsUser.getDisplayString(self.getDisplayKeys(), self.getDisplayFormat())
        print 'APS User Info: ', apsUser
        print 
        ldapUserManager = LdapUserManager.createInstance()
        try:
            self.logger.debug('Modifying LDAP user with badge %s: %s' % (badge, apsUser['name']))
            ldapUser = ldapUserManager.getUserInfo(username)
            print 'LDAP User Info (before update): ', ldapUser
            print 'LDAP User Password Hash (before update): ', ldapUserManager.decodePasswordHash(ldapUser.getUserPassword())
            print
            ldapUserManager.modifyUserInfo(username, apsUser)
            ldapUser = ldapUserManager.getUserInfo(username)
        except ObjectNotFound, ex:
            self.logger.debug('User with badge %s does not exist, creating it' % (badge))
            ldapUser = ldapUserManager.createUserInfo(username, apsUser)
        print 'LDAP User Info (after update): ', ldapUser
        print 'LDAP User Password Hash (after update): ', ldapUserManager.decodePasswordHash(ldapUser.getUserPassword())

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = UpdateUserFromApsDbCli()
    cli.run()
